%%% File    : connection_pool_SUITE.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Test suite for connection_pool
%%% Created : 15 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id$
%%
%% $Log$
%%
%%**
%% 
%% @doc Test suite for connection_pool
%% @end
%%*
-module(connection_pool_SUITE).

-export([all/1, shared/1, exclusive/1]).

-export([init_per_testcase/2, fin_per_testcase/2]).

%% internal exports
-export([dummy_worker/1]).

-include("test_server.hrl").


-define(default_timeout, ?t:minutes(1)).

init_per_testcase(PoolType, Config) when list(Config) ->
  Dog=?t:timetrap(?default_timeout),
  Opts = [{pool_start_len, 1}, {pool_max_len, 5}, {pool_type, PoolType},
	  {supervisor_spec, {simple_one_for_one, 1,  1000}},
	  {worker_spec, {{?MODULE, dummy_worker, [dummy_worker_arg]},
			 transient, 5000, worker, []}}],
  %%?line Ref = myapp:start(),
  case connection_pool:start_link({local, test_pool}, Opts) of
    {ok, Pid} when pid(Pid) ->
      [{watchdog, Dog},{pool, Pid}|Config];
    Error -> 
      ?t:fail(Error)
  end.
fin_per_testcase(_, Config) when list(Config) ->
  Dog=?config(watchdog, Config),
  ?t:timetrap_cancel(Dog),
  Pool = ?config(pool, Config),
  ok = connection_pool:stop(Pool),
  case erlang:whereis(test_pool) of
    undefined -> 
      lists:keydelete(pool,1,Config);
    Pid when pid(Pid) -> 
      ?t:fail("connection_pool did not stop")
  end.

all(suite) -> 
  %% Test specification on test suite level
  [exclusive, shared].

exclusive(doc) ->
  "Testing exclusive pool";
exclusive(suite) ->
  [];
exclusive(Config) ->
  Pool = ?config(pool, Config),
  ?line {ok, W0} = connection_pool:connect(Pool),
  ?line ?t:format("0th conn, pool state: ~p~n", [connection_pool:info(Pool)]),
  ?line {ok, W1} = connection_pool:connect(Pool),
  ?line {ok, dummy_worker_arg} = dummy_worker_get(W1),
  dummy_worker_stop(W1),
  ?line ?t:format("1st conn, pool state: ~p~n", [connection_pool:info(Pool)]),
  ?line {ok, W2} = connection_pool:connect(Pool),
  ?line ?t:format("2nd conn, pool state: ~p~n", [connection_pool:info(Pool)]),
  dummy_worker_stop(W2),
  ?line {ok, W3} = connection_pool:connect(Pool),
  ?line connection_pool:disconnect(W3),
  ?line ?t:format("3d conn, pool state: ~p~n", [connection_pool:info(Pool)]),
  ?line {ok, W3} = connection_pool:connect(Pool),
  ?line ?t:format("4th conn, pool state: ~p~n", [connection_pool:info(Pool)]),

  ?line {ok, W01} = connection_pool:connect(Pool),
  ?line {ok, W02} = connection_pool:connect(Pool),
  ?line {ok, W03} = connection_pool:connect(Pool),
  ?t:sleep(20),
  ?line ?t:format("we've got all 5 connections: ~p~n",
		[connection_pool:info(Pool)]),
  ?line {'EXIT', timeout} = (catch connection_pool:connect(Pool, 100)),
  ?line ?t:format("we exited on timeout: ~p~n",
		  [connection_pool:info(Pool)]),
  {Self, W, Ref} = hd(?config(users, connection_pool:info(Pool))),
  ?line gen_server:cast(Pool, {connect_timeout, Self, Ref}),
  ?line ?t:format("we emulated exit on timeout: ~p~n",
		  [connection_pool:info(Pool)]),
  ?line connection_pool:disconnect(W3),
  ?line connection_pool:disconnect(W0),
  ?line connection_pool:disconnect(W0),

  ?line connection_pool:disconnect(W01),
  ?line connection_pool:disconnect(W02),
  ?line connection_pool:disconnect(W03),

  ?line C1 = dummy_client(Pool, 5),
  ?line ?t:format("client request 5 connections: ~p~n",
		  [connection_pool:info(Pool)]),
  ?t:sleep(20),
  ?line Cs = [dummy_client(Pool) || N <- lists:duplicate(5, 1)],
  ?t:sleep(30),
  ?line ?t:format("5 clients connected: ~p~n",
		  [connection_pool:info(Pool)]),
  ?line (catch connection_pool:connect(Pool, 100)),
  ?line ?t:format("we exited on timeout: ~p~n",
		  [connection_pool:info(Pool)]),
  ?line C1 ! stop,
  ?t:sleep(20),
  ?line ?t:format("5 connections client exited: ~p~n",
		  [connection_pool:info(Pool)]),
  ?line lists:foreach(fun (C) -> C ! {start, self()} end, Cs),
  ?t:sleep(20),
  ?line ?t:format("5 clients exited, pool state: ~p~n",
		  [connection_pool:info(Pool)]),
  ok.

shared(doc) ->
  "Testing shared pool";
shared(suite) ->
  [];
shared(Config) ->
  Pool = ?config(pool, Config),
  ?line {ok, W0} = connection_pool:connect(Pool),
  ?line ?t:format("0th conn, pool state: ~p~n", [connection_pool:info(Pool)]),
  ?line {ok, W1} = connection_pool:connect(Pool),
  ?line {ok, dummy_worker_arg} = dummy_worker_get(W1),
  dummy_worker_stop(W1),
  ?line ?t:format("1st conn, pool state: ~p~n", [connection_pool:info(Pool)]),
  ?line {ok, W2} = connection_pool:connect(Pool),
  ?line ?t:format("2nd conn, pool state: ~p~n", [connection_pool:info(Pool)]),
  dummy_worker_stop(W2),
  ?line {ok, W3} = connection_pool:connect(Pool),
  ?line connection_pool:disconnect(W3),
  ?line ?t:format("3d conn, pool state: ~p~n", [connection_pool:info(Pool)]),
  ?line {ok, W3} = connection_pool:connect(Pool),
  ?line ?t:format("4th conn, pool state: ~p~n", [connection_pool:info(Pool)]),

  ?line {ok, W01} = connection_pool:connect(Pool),
  ?line {ok, W02} = connection_pool:connect(Pool),
  ?line {ok, W03} = connection_pool:connect(Pool),
  ?t:sleep(20),
  ?line ?t:format("we've got all 5 connections: ~p~n",
		[connection_pool:info(Pool)]),
  {Self, W, Ref} = hd(?config(users, connection_pool:info(Pool))),
  ?line gen_server:cast(Pool, {connect_timeout, Self, Ref}),
  ?line ?t:format("we emulated exit on timeout: ~p~n",
		  [connection_pool:info(Pool)]),
  ?line connection_pool:disconnect(W3),
  ?line connection_pool:disconnect(W0),
  ?line connection_pool:disconnect(W0),

  ?line connection_pool:disconnect(W01),
  ?line connection_pool:disconnect(W02),
  ?line connection_pool:disconnect(W03),

  ?line C1 = dummy_client(Pool, 5),
  ?line ?t:format("client request 5 connections: ~p~n",
		  [connection_pool:info(Pool)]),
  ?t:sleep(20),
  ?line Cs = [dummy_client(Pool) || N <- lists:duplicate(5, 1)],
  ?t:sleep(30),
  ?line ?t:format("5 clients connected: ~p~n",
		  [connection_pool:info(Pool)]),
  ?line (catch connection_pool:connect(Pool, 100)),
  ?line ?t:format("we exited on timeout: ~p~n",
		  [connection_pool:info(Pool)]),
  ?line C1 ! stop,
  ?t:sleep(20),
  ?line ?t:format("5 connections client exited: ~p~n",
		  [connection_pool:info(Pool)]),
  ?line lists:foreach(fun (C) -> C ! {start, self()} end, Cs),
  ?t:sleep(100),
  ?line ?t:format("5 clients exited, pool state: ~p~n",
		  [connection_pool:info(Pool)]),
  ok.

%% simple pool worker
dummy_worker(Arg) ->
  Pid = spawn_link(fun () -> dummy_worker_loop(Arg) end),
  {ok, Pid}.

dummy_worker_loop(Arg) ->
  receive
    {get, From} ->
      From ! {value, self(), Arg},
      dummy_worker_loop(Arg);
    {stop, From} ->
      From ! {stopped, self()},
      ok
  end.

dummy_worker_get(Cid) ->
  W = connection_pool:conn(worker, Cid),
  W ! {get, self()},
  receive
    {value, W, V} ->
      {ok, V}
  after 1000 ->
      exit({woker_timeout, W})
  end.

dummy_worker_stop(Cid) ->
  W = connection_pool:conn(worker, Cid),
  W ! {stop, self()},
  receive
    {stopped, W} ->
      ok
  after 1000 ->
      exit({woker_timeout, W})
  end.

dummy_client(Pool) ->
  spawn_link(fun () -> dummy_client_loop(Pool) end).

dummy_client_loop(Pool) ->
  ?line {ok, W1} = connection_pool:connect(Pool),
  receive
    {start, Parent} ->
      ?line {ok, dummy_worker_arg} = dummy_worker_get(W1),
      Parent ! {ok, self()};
    stop ->
      ok
  end.

dummy_client(Pool, Num) ->
  spawn_link(fun () -> dummy_client(Pool, Num, []) end).

dummy_client(Pool, 0, Ws) ->
  receive
    stop ->
      ok
  end;
dummy_client(Pool, Num, Ws) ->
  {ok, W} = connection_pool:connect(Pool),
  dummy_client(Pool, Num-1, [W, Ws]).

% not_started(suite) ->
%   %% Test specification on test case level
%   [not_started_func1, not_started_func2];
% not_started(doc) ->
%   ["Testing all functions when application is not started"].
% %% No execution clause unless the specification clause returns [].

% not_started_func1(suite) ->
%   [];
% not_started_func1(doc) ->
%   ["Testing function 1 when application is not started"].
% not_started_func1(Config) when list(Config) ->
%   ?line {error, not_started} = myapp:func1(dummy_ref,1),
%   ?line {error, not_started} = myapp:func1(dummy_ref,2),
%   ok.

% not_started_func2(suite) ->
%   [];
% not_started_func2(doc) ->
%   ["Testing function 2 when application is not started"].
% not_started_func2(Config) when list(Config) ->
%   ?line {error, not_started} = myapp:func2(dummy_ref,1),
%   ?line {error, not_started} = myapp:func2(dummy_ref,2),
%   ok.


% func1(suite) ->
%   [];
% func1(doc) ->
%   ["Test that func1 returns ok when argument is 1 and error if argument is 2"];
% func1(Config) when list(Config) ->
%   ?line Ref = ?config(myapp_ref,Config),
%   ?line ok = myapp:func1(Ref,1),
%   ?line error = myapp:func1(Ref,2),
%   ok.

% func2(suite) ->
%   [];
% func2(doc) ->
%   ["Test that func1 returns ok when argument is 3 and error if argument is 4"];
% func2(Config) when list(Config) ->
%   ?line Ref = ?config(myapp_ref,Config),
%   ?line ok = myapp:func2(Ref,3),
%   ?line error = myapp:func2(Ref,4),
%   ok.

