%%%----------------------------------------------------------------------
%%% File    : listn.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created : 12 Oct 2001 by  <svg@surnet.ru>
%%%----------------------------------------------------------------------

-module(svglib.listn).
-vsn('$Id\$').
-author('svg@surnet.ru').

-export([listn/1, appendn/1, foldn/3, mapn/2, zip/2, zipn/1, unzip/1, unzipn/1]).
-export([make_list/1]).

-import(.lists).

listn(Ls) ->
  [lists:reverse(L)
   || L <- foldn(fun (A, Acc) -> [A|Acc] end, [], Ls)].

appendn(L) ->
  foldn(fun (A, Acc) -> Acc ++ A end, [], L).

foldn(_, _, []) ->
  [];
foldn(Fun, Acc0, Ls) ->
  foldn(Fun, Acc0, Ls, []).

foldn(_, _, [[]|_], Ret) ->
  lists:reverse(Ret);
foldn(Fun, Acc0, Ls, Ret) ->
  foldn(Fun, Acc0,
	[tl(L) || L <- Ls],
	[lists:foldl(Fun, Acc0, [hd(L) || L <- Ls])|Ret]
       ).

mapn(Fun, []) ->
  [];
mapn(Fun, List) when list(List) ->
  mapn(Fun, List, []).

mapn(Fun, [List], Args) ->
  lists:map(fun (E) -> apply(Fun, lists:reverse([E|Args])) end, List);
mapn(Fun, [List|Rest], Args) ->
  lists:map(fun (E) -> mapn(Fun, Rest, [E|Args]) end, List).

zip(L1, L2) ->
  zip(L1, L2, []).

zip(L1, L2, Res) when L1 == []; L2 == [] ->
  lists:reverse(Res);
zip([E1|Rest1], [E2|Rest2], Acc) ->
  zip(Rest1, Rest2, [{E1, E2}|Acc]).

zipn(List) ->
  [list_to_tuple(L) || L <- listn(List)].


unzip(List) ->
  {L1, L2} =
    lists:foldl(fun ({E1, E2}, {Es1, Es2}) ->
		    {[E1|Es1], [E2|Es2]}
		end, {[], []}, List),
  {lists:reverse(L1), lists:reverse(L2)}.

unzipn(List) ->
  listn([tuple_to_list(T) || T <- List]).

make_list(V) when list(V) ->
  V;
make_list(V) ->
  [V].
