%%%-------------------------------------------------------------------
%%% File    : str.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Some string manipulation functions
%%%
%%% Created : 28 Nov 2002 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%% $Log$
%%
%%**
%% 
%% @doc
%% @end
%%*

-module(svglib.str).

-export([join/2, capitalize/1, to_upper/1, to_lower/1, trim/2, trim/3]).

-import(.lists).
-import(.string).

%%**
%% @spec join(Strings::[string()], Sep::string() ) -> string()
%%
%% @doc Join <code>Strings</code> with <code>Sep</code> separator
%% @end
%%*
join([], _) ->
    [];
join([S|Rest], Sep) ->
  lists:flatten(
    lists:foldl(fun (Str, Acc) ->
		    [Acc|[Sep|Str]]
		end, [S], Rest)).

capitalize([]) ->
  [];
capitalize([C|Rest]) ->
  [char:to_upper(C)|Rest].

%% to_upper
to_upper(Str) when list(Str) ->
  [char:to_upper(C) || C <- Str].

%% to_lower
to_lower(Str) when list(Str) ->
  [char:to_lower(C) || C <- Str].

trim([], _) -> [];
trim(Str, Chrs) ->
  trim(trim(Str, Chrs, left), Chrs, right).

trim([], _, _) -> [];
trim(Str, Chrs, left) ->
  string:substr(Str, max(string:span(Str, Chrs), 1));
trim(Str, Chrs, right) ->
  lists:reverse(trim(lists:reverse(Str), Chrs, left)).

max(A, B) when A > B -> A;
max(_, B) -> B.
